;-----------------------------------------------------------
; title		: capteur de couleur
; files		: couleur.asm, 2313def.inc
; revision	: 1.0
; last update	: 19/10/2003
; target	: at90s2313-10pc
; crystal	: 7.3728mhz
; author	: yann leidwanger
; email		: yann.leidwanger@wanadoo.fr
; url		: http://microbot.free.fr
;-----------------------------------------------------------

.include "2313def.inc"

;-----------------------------------------------------------
; definitions
;-----------------------------------------------------------
.def	temp		= r16		;mcu
.def	temp2		= r17
.def	cpt		= r18		;wait sub
.def	car		= r19		;uart car

.def	id_coul		= r23		;id couleur

.equ	led_red		= pb6		;red led
.equ	led_blue	= pb5		;blue led
.equ	led_green	= pb7		;greeb led

.equ	config		= pd5		;auto start

.equ	port_can	= portb		;adc port
.equ	pin_can		= pinb
.equ	cs		= pb2		;adc pins
.equ	clk		= pb3
.equ	dout		= pb4

.equ	cr		= $0d
.equ	lf		= $0a
.equ	null		= $00		;ascii codes
.equ	eot		= $04

;-----------------------------------------------------------
; code segment
;-----------------------------------------------------------
.cseg
.org	$0000
	rjmp	reset			;reset
	rjmp	no_isr			;int0
	rjmp	no_isr			;int1
	rjmp	no_isr			;t1 capture
	rjmp	no_isr			;t1 compare
	rjmp	t1_isr			;t1 overflow
	rjmp	no_isr			;t0 overflow
	rjmp	rx_isr			;rx uart
	rjmp	no_isr			;udr uart
	rjmp	no_isr			;tx uart
	rjmp	no_isr			;comparator
no_isr:
	reti

;-----------------------------------------------------------
; system init
;-----------------------------------------------------------
init_syst:
	ldi	temp,$ec
	out	ddrb,temp		;portb i/o

	ldi	temp,$0c
	out	portb,temp		;init outputs

	ldi	temp,$00
	out	ddrd,temp		;portd i/o

	ldi	temp,$20
	out	portd,temp		;pullup res + init

	clr	car
	clr	id_coul

	ret

;-----------------------------------------------------------
; uart init
;-----------------------------------------------------------
uart_init:
	ldi	temp,$98	;rxien, rx, tx
	out	ucr,temp
	ldi	temp,$17	;19200 bauds @ 7.3728mhz
	out	ubrr,temp
	ret

;-----------------------------------------------------------
; uart isr
;-----------------------------------------------------------
rx_isr:
	in	car,udr		;ack uart
	reti

;-----------------------------------------------------------
; emission uart ; data dans temp
;-----------------------------------------------------------
uart_emit:
	sbis	usr,udre	;udr empty ?
	rjmp	uart_emit
	out	udr,temp	;emit data
	ret

;-----------------------------------------------------------
; string(z)->uart
;-----------------------------------------------------------
.macro	_string_uart
	ldi	zh,high(@0*2)	;z pointer init
	ldi	zl,low(@0*2)
	rcall	string_uart
.endmacro

string_uart:
	lpm
	mov	temp,r0
	cpi	temp,eot	;eot test
	breq	string_uart_end
	rcall	uart_emit	;write data to uart
	adiw	zl,1
	rjmp	string_uart
string_uart_end:
	ret

;-----------------------------------------------------------
; hexa to ascii -> uart, data in temp
;-----------------------------------------------------------
h2a_uart_byte:			;convert byte
	push	temp
	swap	temp
	rcall	h2a_uart_nibble	;convert h_nibble
	pop	temp
h2a_uart_nibble:		;convert l_nibble
	andi	temp,$0f
	ldi	temp2,$f6
	add	temp,temp2
	brcc	h2a_uart_nibble2
	ldi	temp2,$07
	add	temp,temp2
h2a_uart_nibble2:
	ldi	temp2,$3a
	add	temp,temp2
	rcall	uart_emit
	ret

;-----------------------------------------------------------
; timer1 init
;-----------------------------------------------------------
t1_init:
	ldi	temp,$fc
	out	tcnt1h,temp
	ldi	temp,$df
	out	tcnt1l,temp	;t1 init
	ldi	temp,$05
 	out	tccr1b,temp	;init timer1 : ck / 1024
	ldi	temp,$80
	out	timsk,temp	;enable timer1 int
	ret

t1_on:
	ldi	temp,$80
	out	timsk,temp	;enable timer1 int
	ret

t1_off:
	ldi	temp,$00
	out	timsk,temp	;disable timer1 int
	ret

;-----------------------------------------------------------
; timer1 interrupt service routine
;-----------------------------------------------------------
; timer1 config
; tcnt1 = 65535 - (rt * ck / 1024)
; $fcdf -> 111ms
;-----------------------------------------------------------
t1_isr:
	push	temp
	in	temp,sreg
	push	temp		;save cpu context

	ldi	temp,$fc
	out	tcnt1h,temp
	ldi	temp,$df
	out	tcnt1l,temp	;t1 re-init

	cpi	id_coul,0
	breq	acq_red		;red led ?
	cpi	id_coul,1
	breq	acq_blue	;blue led ?
	cpi	id_coul,2
	breq	acq_green	;green led ?
	clr	id_coul
	rjmp	end_t1_isr	;else end

acq_red:
	cbi	portb,led_green		;led green off
	sbi	portb,led_red		;led red on
	ldi	temp,'R'
	rcall	uart_emit		;message
	rcall	read_adc		;read adc
	rcall	h2a_uart_byte		;emit measure
	ldi	id_coul,1
	rjmp	end_t1_isr		;end

acq_blue:
	cbi	portb,led_red		;led red off
	sbi	portb,led_blue		;led blue on
	ldi	temp,'B'
	rcall	uart_emit		;message
	rcall	read_adc		;read adc
	rcall	h2a_uart_byte		;emit measure
	ldi	id_coul,2
	rjmp	end_t1_isr		;end

acq_green:
	cbi	portb,led_blue		;led blue off
	sbi	portb,led_green		;led green on
	ldi	temp,'G'
	rcall	uart_emit		;message
	rcall	read_adc		;read adc
	rcall	h2a_uart_byte		;emit measure
	clr	id_coul

	ldi	temp,cr
	rcall	uart_emit
	ldi	temp,lf
	rcall	uart_emit		;print cr, lf

end_t1_isr:
	pop	temp
	out	sreg,temp
	pop	temp			;restore cpu context
	reti

;-----------------------------------------------------------
; read adc, result in temp
;-----------------------------------------------------------
read_adc:
	cbi	port_can,cs
	nop				;tsu can (from data-sheet)
	nop
	nop
	nop
	ldi	cpt,11			;# of bits
loop_can:
	cbi	port_can,clk
	sbis	pin_can,dout		;dout test
	rjmp	in1_0
in1_1:
	sec				;bit = 1
	rol	temp
	sbi	port_can,clk
	dec	cpt
	brne	loop_can
	rjmp	end_can
in1_0:
	clc				;bit = 0
	rol	temp
	sbi	port_can,clk
	dec	cpt
	brne	loop_can
end_can:
	sbi	port_can,cs
	ret

;-----------------------------------------------------------
; main program
;-----------------------------------------------------------
reset:
	ldi	temp,low(ramend)
	out	spl,temp		;sp init

	rcall	init_syst		;system init
	rcall	uart_init		;uart init

	sbis	pind,config
	rjmp	auto_start		;auto start

	_string_uart	msg1
	_string_uart	msg2		;print start messages
	_string_uart	msg_prompt	;print prompt

	rcall	t1_init			;t1 init

	sei				;enable global int

	rcall	t1_off			;stop continupus measures

main:
	tst	car
	breq	main			;received car ?
menu:
	cpi	car,'m'
	breq	menu_1			;start continuous measures
	cpi	car,'a'
	breq	menu_2			;stop continuous measures
	clr	car			;ack received car
	_string_uart	msg_prompt	;print prompt
	rjmp	main			;main loop

menu_1:
	_string_uart	msg3
	rcall	t1_on			;enable timer1
	clr	car			;ack received car
	rjmp	main			;main loop

menu_2:
	_string_uart	msg4
	rcall	t1_off			;disable timer1
	cbi	portb,led_red
	cbi	portb,led_blue
	cbi	portb,led_green		;leds off
	clr	car			;ack received car
	_string_uart	msg_prompt	;print prompt
	rjmp	main			;main loop



auto_start:
	rcall	t1_init			;t1 init
	sei				;enable global int

main_auto:
	rjmp	main_auto

;-----------------------------------------------------------
; constants in flash
;-----------------------------------------------------------
.org	0x0300

msg_prompt:
	.db	cr,lf,">"
	.db	eot

msg1:
	.db	cr,lf,"Capteur de couleur"
	.db	cr,lf,"Micros et Robots  Y. Leidwanger  oct. 2003"
	.db	cr,lf,eot

msg2:
	.db	cr,lf,"m   : mesure continue : marche"
	.db	cr,lf,"a   : mesure continue : arret"
	.db	cr,lf,eot
msg3:
	.db	cr,lf,"Mesures continues en cours ..."
	.db	cr,lf,eot
msg4:
	.db	cr,lf,"Mesures continues arretees"
	.db	cr,lf,eot
msg5:

;-----------------------------------------------------------
; that's all folks !!!
;-----------------------------------------------------------
